/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <inttypes.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <zephyr/sys/util.h>
#include <zephyr/sys/printk.h>
#include "error.h"
#include "led.h"
#include "pwr_mgmt.h"
#define BUTTON_TIMER_MSEC 10000

/*
 * Get configuration from the devicetree alias. This is mandatory.
 */
#define BOOT_BUTTON_NODE DT_ALIAS(sw0)

#if !DT_NODE_HAS_STATUS(BOOT_BUTTON_NODE, okay)
#error "Unsupported board: mcuboot-button0 devicetree alias is not defined"
#endif

LOG_MODULE_REGISTER(button);

typedef enum {
    BUTTON_RELEASE = 0,
    BUTTON_PRESS
} button_state_t;

/*
 * Helper macro for initializing a gpio_dt_spec from the devicetree
 * with fallback values when the nodes are missing.
 */
static const struct gpio_dt_spec _gButton   = GPIO_DT_SPEC_GET_OR(BOOT_BUTTON_NODE, gpios, {0});

static struct gpio_callback button_cb_data;

static void _buttonHandler(struct k_timer *dummy) {
    printk("%" PRId64 "\n", k_uptime_get());
    int val = gpio_pin_get_dt(&_gButton);
    if (val == BUTTON_PRESS) {
        const power_state_t state = pwrMgmtGetPowerState();
        if (state == POWER_DOWN) {
            printk("power up\n");
            pwrMgmtPowerUp();
            ledSetG();
        } else {
            printk("power down\n");
            pwrMgmtPowerDown();
        }
    }
}

K_TIMER_DEFINE(_gButtonTimer, _buttonHandler, NULL);


void button_pressed(const struct device *dev, struct gpio_callback *cb, uint32_t pins) {
    printk("Button pressed at %" PRIu32 "\n", k_cycle_get_32());
    printk("start a timer\n");
    //ltPrintCurrentDatetime();
    printk("%" PRId64 "\n", k_uptime_get());
    k_timer_start(&_gButtonTimer, K_MSEC(BUTTON_TIMER_MSEC), K_NO_WAIT);
}

int buttonInit(void) {
    LOG_INF("button long press test\n");
    printk("Starting button Test example\n");
    int ret;
    
    ledInit();

    if (!device_is_ready(_gButton.port)) {
        printk("Error: button device %s is not ready\n", _gButton.port->name);
        return ERROR_BT_INITIALIZE_FAIL;
    }

    printk("device_is_ready() ok\n");

    ret = gpio_pin_configure_dt(&_gButton, GPIO_INPUT);
    if (ret != 0) {
        printk("Error %d: failed to configure %s pin %d\n",
               ret, _gButton.port->name, _gButton.pin);
        return ERROR_BT_INITIALIZE_FAIL;
    }
    
    printk("gpio_pin_configure_dt() ok\n");

    ret = gpio_pin_interrupt_configure_dt(&_gButton, GPIO_INT_EDGE_TO_ACTIVE);
    if (ret != 0) {
        printk("Error %d: failed to configure interrupt on %s pin %d\n",
               ret, _gButton.port->name, _gButton.pin);
        return ERROR_BT_INITIALIZE_FAIL;
    }

    printk("gpio_pin_interrupt_configure_dt() ok\n");

    gpio_init_callback(&button_cb_data, button_pressed, BIT(_gButton.pin));
    gpio_add_callback(_gButton.port, &button_cb_data);
    printk("Set up button at %s pin %d\n", _gButton.port->name, _gButton.pin);
    
    int val = gpio_pin_get_dt(&_gButton);
    printk("button state=%d\n", val);
    if (val == BUTTON_PRESS) {
        printk("start a timer\n");
       // ltPrintCurrentDatetime();
        printk("%" PRId64 "\n", k_uptime_get());
        k_timer_start(&_gButtonTimer, K_MSEC(BUTTON_TIMER_MSEC), K_NO_WAIT);
    }

    return 0;
}
